#!/usr/bin/perl

#  Copyright 2003 by Fred Van Andel (fred@vanandel.net)
#
#  Licenced under the GPL
#
#  Extract USE flags information from the /usr/portage/ directory
#  and print lists to stdout showing how they are used.
#
#  Note: Sometimes different versions of a package will have 
#        different USE flags, this software will show them all
#        under the base package name.
#


use File::Find;
use strict vars;

our(%Packages, %UseFlags);

sub ProcessEachFile 
   {
      our($Pname, @UseList, $tmp, $i, $last);
      
      if(($File::Find::name =~ m!\.ebuild$!) != 0)
         {
            open(EBUILD, "<" . $File::Find::name);
            $/ = '<<NON EXISTENT SEPARATOR>>';   # Set slurp mode
            $_ = <EBUILD>; 
            close(EBUILD);
        
            # extract the use flags from the IUSE variable 
            m!IUSE[ \t]*=[ \t]*"([^"]*)"!ig;
            $tmp = $1;
	         $tmp =~ s!\n! !;
            @UseList = split(m![ ]+!, $tmp);

            # produce the package name from the file name            
            if($File::Find::name =~ m!^/usr/portage/([^/]*/[^/]*)/! != 0)
               { $Pname = $1; }
            else
	            { $Pname = ""; }     

            # if we have use flags and a package name then save them for later processing
            if(($#UseList != -1) && ($Pname ne ""))
               {
                  $last = "";
                  for $i (sort(@UseList))
                    {  
                       if($last ne $i) 
                          {
                             $last = $i;
                             $Packages{$i} .= "$Pname\t"; 
                             $UseFlags{$Pname} .= "$i\t";
                          }
                    }
               } 

         }
   }


find(\&ProcessEachFile, ("/usr/portage/"));

our(@FlagList, @UseList, $i, $j, $last);

# print out the list of use flags for each package
for $i (sort(keys %UseFlags))
   {
      
      @FlagList = sort(split(m!\t!, $UseFlags{$i}));
      print "$i:";
      $last = "";
      for $j (@FlagList)
         { 
            if($last ne $j)
               {
                  print " $j";
                  $last = $j;
               }	
         }
      print "\n";           
   } 

print "\n\n\n";

# print out the packages for each use flag
for $i (sort(keys %Packages))
   {
      our(@UseList, $i, $j, $last);
      
      @UseList = sort(split(m!\t!, $Packages{$i}));
      print "$i:\n";
      $last = "";
      for $j (@UseList)
         { 
            if($last ne $j)
               {
                  print "    $j\n";
                  $last = $j;
               }		  
         }
           
   } 
